/*
 * Classes:
 *  CVerticalResizer
 *  CHorizontalResizer
 *  CSelection
 *  CToolButton
 *  CToolBar
 *  CFolderParams
 *  CFolderLine
 *  CSpaceInfo
 *  CContactCard
 *  CMessageCharsetSelector
 *  CAddToAddressBookImg
 *  PreviewPaneMessageHeaders
 */

var VR_TYPE_INBOX = 0;
var VR_TYPE_MESS = 1;
var VR_TYPE_HEAD = 2;

function CVerticalResizer(DIVMovable, parentTable, divHSize, minLeftWidth, minRightWidth, leftPosition, endMoveHandler, type) {
	// set internal data by outside parameters
	this._type = (type) ? type : VR_TYPE_INBOX;
	switch (this._type) {
		case VR_TYPE_MESS:
			this._class = 'wm_vresizer_mess';
			this._classPress = 'wm_vresizer_mess';
			break;
		case VR_TYPE_HEAD:
			this._class = 'wm_inbox_headers_separate';
			this._classPress = 'wm_inbox_headers_separate';
			break;
		default:
			this._type = VR_TYPE_INBOX;
			this._class = 'wm_vresizer';
			this._classPress = 'wm_vresizer_press';
			break;
	}
	this._DIVMovable = DIVMovable;
	this._parentTable = parentTable;
	this._divHSize = divHSize;
	this._minLeftWidth = minLeftWidth;
	this._minRightWidth = minRightWidth;
	this._leftPosition = leftPosition;
	this._beginPosition = 0;
	this._endMoveHandler = endMoveHandler;

	// set some internal data by default values (this values must be overwritten)
	this._leftBorder = 0;
	this._rightBorder = 600;
	this._leftLimit = 80;
	this._rightLimit = 550;

	this._divVSize = 2;
	this._divVSizePress = 2;
	
	with (this._DIVMovable.style) {
		width = this._divHSize + 'px';
		height = this._divVSize + 'px';
		cursor = 'e-resize';
	}
	switch (this._type) {
		case VR_TYPE_INBOX:
			this._leftShear = 0;
			this._DIVMovable.style.left = '1px';
			break;
		case VR_TYPE_MESS:
		case VR_TYPE_HEAD:
			this._leftShear = leftPosition;
			break;
	}
	this._DIVMovable.className = this._class;
	if (this._type != VR_TYPE_HEAD) {
		this._DIVMovable.innerHTML = '&nbsp;';
	}
	
	// this handler is necessary to begins moving
	var obj = this;
	this._DIVMovable.onmousedown = function (e) {
		obj.beginMoving(e);
		return false; //don't select content in Opera
	}
}

CVerticalResizer.prototype = {
	updateVerticalSize: function (vert_size, vert_size_press)
	{
		this._divVSize = vert_size;
		this._DIVMovable.style.height = this._divVSize + 'px';
		this._divVSizePress = (this._type == VR_TYPE_HEAD) ? vert_size_press : vert_size;
	},
	
	updateMinLeftWidth: function (minLeftWidth)
	{
		this._minLeftWidth = minLeftWidth;
	},
	
	updateMinRightWidth: function (minRightWidth)
	{
		this._minRightWidth = minRightWidth;
	},
	
	updateLeftPosition: function (leftPosition)
	{
		var diff = leftPosition - this._leftPosition;
		this._minLeftWidth += diff;
		this._leftPosition = leftPosition;
		this._DIVMovable.style.left = leftPosition + 'px';
	},

	beginMoving: function (e)
	{
   		e = e ? e : event;
		this._beginPosition = e.clientX;
		this._DIVMovable.className = this._classPress;
		if (this._type == VR_TYPE_HEAD) {
			this._DIVMovable.style.height = this._divVSizePress + 'px';
		}
		//don't select content in IE
		document.onselectstart = function() {return false;};
		document.onselect = function() {return false;};
		
		// calculate borders of this._parentTable
		var bounds = GetBounds(this._parentTable);
		this._leftBorder = bounds.Left;
		this._rightBorder = bounds.Left + bounds.Width;

		// calculate moving limits (for center of movable td/div)
		if (window.RTL && this._type == VR_TYPE_INBOX) {
			this._leftLimit = this._leftBorder  + this._minRightWidth;
			this._rightLimit = this._rightBorder - this._minLeftWidth;
		}
		else {
			this._leftLimit = this._leftBorder  + this._minLeftWidth + (this._beginPosition - this._leftPosition) - this._leftBorder;
			this._rightLimit = this._rightBorder - this._minRightWidth - ((this._leftPosition + 6) - this._beginPosition) - this._leftBorder;
		}

		// hang moving handlers	
		var obj = this;
		this._parentTable.onmousemove = function (e) {
		    if (arguments.length == 0) e = event;
			obj.processMoving( e.clientX ); 
		};
		
		this._parentTable.onmouseup = function () {
			obj.endMoving();
		};
		
		this._parentTable.onmouseout = function (e) {
		    if (arguments.length == 0) e = event;

        	var b = GetBounds(this);
			var left_border = b.Left;
			var top_border = b.Top;
			var right_border = left_border + b.Width;
			var bottom_border = top_border + b.Height;
			
			// it is necessary to prevent incorrect action on mouseout event
			if( e.clientX<=left_border || e.clientX>=right_border ||
			 e.clientY<=top_border || e.clientY>=bottom_border ) {
				obj.endMoving();
			}
		}
	},
	
	processMoving: function (mouse_x)	
	{
		// check and correct mouse_x if it is necessary
		if( mouse_x < this._leftLimit ){
			mouse_x = this._leftLimit;
		}
		if( mouse_x > this._rightLimit ){
			mouse_x = this._rightLimit;
		}
		switch (this._type) {
			case VR_TYPE_INBOX:
				this._DIVMovable.style.left = mouse_x - this._beginPosition + 1 + 'px';
				this._leftShear = mouse_x - this._beginPosition;
				break;
			case VR_TYPE_MESS:
				var new_left = this._leftPosition + mouse_x - this._beginPosition;
				if (new_left < (this._leftLimit - (this._beginPosition - this._leftPosition))) {
					new_left = this._leftLimit - (this._beginPosition - this._leftPosition);
				}
				if (new_left > this._rightLimit) {
					new_left = this._rightLimit + ((this._leftPosition + 6) - this._beginPosition);
				}
				this._leftShear = new_left;
				eval(this._endMoveHandler);
				break;
			case VR_TYPE_HEAD:
				var new_left = this._leftPosition + mouse_x - this._beginPosition;
				if (new_left < (this._leftLimit - (this._beginPosition - this._leftPosition))) {
					new_left = this._leftLimit - (this._beginPosition - this._leftPosition);
				}
				if (new_left > this._rightLimit) {
					new_left = this._rightLimit + ((this._leftPosition + 6) - this._beginPosition);
				}
				this._DIVMovable.style.left = new_left + 'px';
				this._leftShear = new_left;
				break;
		}
	},
	
	endMoving: function ()
	{
		document.onselectstart = function() {};
		document.onselect = function() {};
		this._parentTable.onmousemove = '';
		this._parentTable.onmouseup = '';
		this._parentTable.onmouseout = '';
		this._DIVMovable.className = this._class;
		switch (this._type) {
			case VR_TYPE_INBOX:
				this._DIVMovable.style.left = '1px';
				var new_left = this._leftPosition;
				if (window.RTL) {
					new_left -= this._leftShear;
				}
				else {
					new_left += this._leftShear;
					if (new_left < (this._leftLimit - (this._beginPosition - this._leftPosition))) {
						new_left = this._leftLimit - (this._beginPosition - this._leftPosition);
					}
					if (new_left > this._rightLimit) {
						new_left = this._rightLimit + ((this._leftPosition + 6) - this._beginPosition);
					}
				}
				this._leftPosition = new_left;
				this._leftShear = 0;
				eval(this._endMoveHandler);
				break;
			case VR_TYPE_MESS:
				this._leftPosition = this._leftShear;
				break;
			case VR_TYPE_HEAD:
				this._DIVMovable.style.height = this._divVSize + 'px';
				this._leftPosition = this._leftShear;
				eval(this._endMoveHandler);
				break;
		}
	},
	
	free: function ()
	{
		this._parentTable.onmousemove = '';
		this._parentTable.onmouseup = '';
		this._parentTable.onmouseout = '';
		this._DIVMovable.onmousedown = '';
		this._DIVMovable.style.cursor = 'default';
	},
	
	busy: function (width)
	{
		this._leftPosition = width;
		
		this._DIVMovable.style.cursor = 'e-resize';
		
		// this handler is necessary to begins moving
		var obj = this;
		this._DIVMovable.onmousedown = function(e) {
			obj.beginMoving(e);
			return false; //don't select content in Opera
		};
	}
};

function CHorizontalResizer(DIVMovable, parentTable, divVSize, minUpperHeight, minLowerHeight, topPosition, endMoveHandler) {
	// set internal data by outside parameters
	this._DIVMovable = DIVMovable;
	this._parentTable = parentTable;// table (HTML Element) which contents all changable TRs
	this._divVSize = divVSize;// vertical size of movable TR/TD/DIV
	this._minUpperHeight = minUpperHeight;// minimal height when upper TR has good look
	this._minLowerHeight = minLowerHeight;// minimal height when lower TR has good look
	this._topPosition = topPosition;
	this._topShear = 0;
	this._beginPosition = 0;
	this._endMoveHandler = endMoveHandler;

	this._class = 'wm_hresizer';
	this._classPress = 'wm_hresizer_press';
	
	// set some internal data by default values (this values must be overwritten)
	this._upperBorder = 114;
	this._lowerBorder = 815;
	this._upperLimit = 268;
	this._lowerLimit = 665;

	this._divHSize = 2	;

	with (this._DIVMovable.style) {
		width = this._divHSize + 'px';
		height = this._divVSize + 'px';
		cursor = 's-resize';
		top = '0px';
	}
	this._DIVMovable.className = this._class;
	this._DIVMovable.innerHTML = '&nbsp;';

	// this handler is necessary to begins moving
	var obj = this;
	this._DIVMovable.onmousedown = function(e) {
		obj.beginMoving(e);
		return false; //don't select content in Opera
	};
}

CHorizontalResizer.prototype = 
{
	updateHorizontalSize: function (horiz_size)
	{
		this._divHSize = horiz_size;
		this._DIVMovable.style.width = this._divHSize + 'px';
	},

	beginMoving: function (e)
	{
   		e = e ? e : event;
		this._beginPosition = e.clientY;
		this._DIVMovable.className = this._classPress;
		//don't select content in IE
		document.onselectstart = function() {return false;};
		document.onselect = function() {return false;};

		// calculate borders of this._parentTable
		var bounds = GetBounds(this._parentTable);
		this._upperBorder = bounds.Top;
		this._lowerBorder = bounds.Top + bounds.Height;

		// calculate moving limits (for center of movable td/div)
		this._upperLimit = this._upperBorder + this._minUpperHeight + (this._beginPosition - this._topPosition) - this._upperBorder;
		this._lowerLimit = this._lowerBorder - this._minLowerHeight - ((this._topPosition + 6) - this._beginPosition) - this._upperBorder;

		// hang moving handlers	
		var obj = this;
		this._parentTable.onmousemove = function(e) {
		    if (arguments.length == 0) e = event;
			obj.processMoving( e.clientY ); 
		};

		this._parentTable.onmouseup = function() {
			obj.endMoving();
		};

		this._parentTable.onmouseout = function(e) {
		    if (arguments.length == 0) e = event;

        	var b = GetBounds(this);
			var left_border = b.Left;
			var top_border = b.Top;
			var right_border = left_border + b.Width;
			var bottom_border = top_border  + b.Height;
			
			// it is necessary to prevent incorrect action on mouseout event
			if (e.clientX <= left_border || e.clientX >= right_border ||
				e.clientY <= top_border || e.clientY >= bottom_border) {
				obj.endMoving();
			}
		}
	},

	processMoving: function (mouse_y)	
	{
		// check and correct mouse_y if it is necessary
		if (mouse_y < this._upperLimit) {
			mouse_y = this._upperLimit;
		}
		if (mouse_y > this._lowerLimit) {
			mouse_y = this._lowerLimit;
		}
		this._DIVMovable.style.top = mouse_y - this._beginPosition + 'px';
		this._topShear = mouse_y - this._beginPosition;
	},
	
	endMoving: function ()
	{
		this._DIVMovable.className = this._class;
		this._DIVMovable.style.top = '0px';
		document.onselectstart = function() {};
		document.onselect = function() {};
		var new_top = this._topPosition + this._topShear;
		if (new_top < (this._upperLimit - (this._beginPosition - this._topPosition))) {
			new_top = this._upperLimit - (this._beginPosition - this._topPosition);
		}
		if (new_top > this._lowerLimit + ((this._topPosition + 6) - this._beginPosition)) {
			new_top = this._lowerLimit + ((this._topPosition + 6) - this._beginPosition);
		}
		this._topPosition = new_top;
		this._topShear = 0;
		this._parentTable.onmousemove = '';
		this._parentTable.onmouseup = '';
		this._parentTable.onmouseout = '';
		eval(this._endMoveHandler);
	},
	
	free: function ()
	{
		this._parentTable.onmousemove = '';
		this._parentTable.onmouseup = '';
		this._parentTable.onmouseout = '';
		this._DIVMovable.onmousedown = '';
		this._DIVMovable.style.cursor = 'default';
	}
};

function CSelection(fillSelectedContactsHandler)
{
	this._fillSelectedContactsHandler = null;
	if (fillSelectedContactsHandler)
		this._fillSelectedContactsHandler = fillSelectedContactsHandler;
	this._lines = Array();
	this._length = 0;
	this._currIdx = -1;
	this._lastCheckIdx = -1;
	this._checkbox = null;
}

CSelection.prototype = 
{
	SetCheckBox: function (checkbox)
	{
		this._checkbox = checkbox;
	},
	
	Free: function ()
	{
		this._lines = Array();
		this._length = 0;
		this._currIdx = -1;
		this._lastCheckIdx = -1;
	},
	
	AddLine: function (line)
	{
		this._lines.push(line);
		this._length = this._lines.length;
	},
	
	SetParams: function (idArray, field, value, isAllMess)
	{
		var readed = 0;
		if (isAllMess) {
			for (var i = this._length-1; i >= 0; i--) {
				var line = this._lines[i];
				readed += line.SetParams(field, value);
			}
		}
		else {
			for (var j in idArray) {
				for (var i = this._length-1; i >= 0; i--) {
					var line = this._lines[i];
					if (line.Id == idArray[j]) {
						readed += line.SetParams(field, value);
						break;
					}
				}
			}
		}
		return readed;
	},
	
	ChangeLineId: function (msg, newId)
	{
		for (var i = this._length-1; i >= 0; i--) {
			var line = this._lines[i];
			if (line.MsgId == msg.Id && line.MsgUid == msg.Uid && line.MsgFolderId == msg.FolderId &&
			 line.MsgFolderFullName == msg.FolderFullName) {
				if (newId) {
					line.Id = newId;
					line.Node.id = newId;
				};
				line.MsgFromAddr = msg.FromAddr;
				line.MsgSubject = msg.Subject;
				line.ApplyFromSubj();
			}
		}
	},
	
	GetCheckedLines: function ()
	{
		var idArray = Array();
		var unreaded = 0;
		for (var i = this._length-1; i >= 0; i--) {
			var line = this._lines[i];
			if (line.Checked == true) {
				if (!line.Read)
					unreaded++;
				idArray.push(line.Id);
			}
		};
		return {IdArray: idArray, Unreaded: unreaded};
	},
	
	CheckCtrlLine: function(id)
	{
		for (var i = this._length-1; i >= 0; i--) {
			var line = this._lines[i];
			if (line.Id == id){
				if (line.Checked == false) {
					line.Check();
					this._currIdx = i;
					this._lastCheckIdx = i;
				}
				else {
					line.Uncheck();
				}
			}
		};
		this.ReCheckAllBox();
	},
	
	CheckLine: function(id)
	{
		for (var i = this._length-1; i >= 0; i--) {
			var line = this._lines[i];
			if (line.Id == id) {
				line.Check();
				this._currIdx = i;
				this._lastCheckIdx = i;
			}
			else {
				line.Uncheck();
			}
		};
		this.ReCheckAllBox();
	},
	
	GetLineIndex: function ()
	{
		return this._lastCheckIdx;
	},
	
	_readyKeysCheckLines: function ()
	{
		return ((this._length != 0) && (this._lastCheckIdx >= 0));
	},
	
	GetFirstLineId: function ()
	{
		if (!this._readyKeysCheckLines()) return -1;
		return this._lines[0].Id
	},
	
	GetNextLineId: function(count)
	{
		if (!this._readyKeysCheckLines()) return -1;
		if ((this._lastCheckIdx + 1) >= this._length) return -1;
		if (!count) count = 1;
		var idx = this._lastCheckIdx + count;
		if (idx >=this._length) idx = this._length - 1;
		return this._lines[idx].Id
	},
	
	GetPrevLineId: function(count)
	{
		if (!this._readyKeysCheckLines()) return -1;
		if (!count) count = 1;
		var idx = this._lastCheckIdx - count;
		if (idx < 0) idx = 0;
		return this._lines[idx].Id;
	},
	
	GetLastLineId: function ()
	{
		if (!this._readyKeysCheckLines()) return -1;
		return this._lines[this._length - 1].Id
	},
	
	GetCurrentTr: function ()
	{
		if (this._currIdx < 0 || this._currIdx >= this._length) return null;
		return this._lines[this._currIdx].Node;
	},

	DragItemsNumber: function (id)
	{
		var findLine = null;
		var number = 0;
		for (var i = this._length-1; i >= 0; i--) {
			var line = this._lines[i];
			if (line.Id == id){
				findLine = line;
			};
			if (line.Checked) {
				number++;
			}
		};
		if (null == findLine) {
			return 0;
		}
		else if (findLine.Checked) {
			return number;
		}
		else {
			this.CheckLine(id);
			return 1;
		}
	},
	
	FlagLine: function (id)
	{
		for (var i = this._length-1; i >= 0; i--) {
			var line = this._lines[i];
			if (line.Id == id){
				if (line.Flagged) {
					line.Unflag();
				} else {
					line.Flag();
				}
			}
		}
	},
	
	CheckShiftLine: function(id)
	{
		if (this._currIdx == -1) {
			this.CheckLine(id);
		}
		else {
			var isChecking = false;
			this._lastCheckIdx = this._currIdx;
			for (var i = 0; i < this._length; i++) {
				var line = this._lines[i];
				if (this._currIdx == i || line.Id == id)
					isChecking = isChecking ? false : true;
				if (line.Id == id)
					this._lastCheckIdx = i;
				if (isChecking || this._currIdx == i || line.Id == id) {
					line.Check();
				}
				else {
					line.Uncheck();
				};
				if (this._currIdx == i && line.Id == id)
					isChecking = isChecking ? false : true;
			}
		};
		this.ReCheckAllBox();
	},
	
	CheckAll: function ()
	{
		for (var i = this._length-1; i >= 0; i--) {
			this._lines[i].Check();
		}
		this._currIdx = -1;
		this._lastCheckIdx = -1;
		this.CheckSelectedContacts();
	},
	
	UncheckAll: function ()
	{
		for (var i = this._length-1; i >= 0; i--) {
			this._lines[i].Uncheck();
		}
		this._currIdx = -1;
		this._lastCheckIdx = -1;
		this.CheckSelectedContacts();
	},
	
	ReCheckAllBox: function()
	{
		var isAllCheck = true;
		for (var i = this._length-1; i >= 0; i--) {
			if (this._lines[i].Checked == false) {
				isAllCheck = false;
				break;
			}
		}
		this.SetCheckboxChecked(isAllCheck);
		this.CheckSelectedContacts();
	},
	
	CheckSelectedContacts: function ()
	{
		if (this._fillSelectedContactsHandler != null) {
			var ContactsArray = Array();
			for (var i = 0; i < this._length; i++) {
				var line = this._lines[i];
				if (line.Checked) {
					ContactsArray.push(line.Node.Email);
				}
			}
			var CurrId = -1;
			var CurrIsGroup = false;
			if (this._currIdx != -1) {
				line = this._lines[this._currIdx];
				CurrId = line.ContactId;
				CurrIsGroup = (line.IsGroup ? true : false);
			}
			this._fillSelectedContactsHandler.call({ContactsArray: ContactsArray, CurrId: CurrId, CurrIsGroup: CurrIsGroup});
		}
	},
	
	SetCheckboxChecked: function (checkedValue)
	{
		if (null != this._checkbox) {
			this._checkbox.checked = checkedValue;
		}
	}
};

function CreateToolBarItemClick(type)
{
	return function () { RequestMessagesOperationHandler(type, []); }
}

function CreateReplyClick(type)
{
	return function () {
		WebMail.ReplyClick(type);
	}
}

function CToolButton(container, desc, handler, show)
{
	this.Cont = container;
	this.Icon = null;
	this._text = null;
	this._hover = false;
	this._className = 'wm_toolbar_item';
	this._classNameOver = 'wm_toolbar_item_over';
	this._build(container, desc);
	this.ChangeHandler(handler);
	if (show) this.Show();
	else this.Hide();
}

CToolButton.prototype = {
	Show: function ()
	{
		var cn = this._className;
		var cno = this._classNameOver;
		if (this._hover) {
			this.Cont.onmouseover = function() { this.className = cno; };
			this.Cont.onmouseout = function() { this.className = cn; };
		}
		this.Cont.className = cn;
	},

	Hide: function ()
	{
		if (this._hover) {
			this.Cont.onmouseover = function () { };
			this.Cont.onmouseout = function () { };
		}
		this.Cont.className = 'wm_hide';
	},
	
	ShowText: function ()
	{
		if (this._text == null) return;
		this._text.className = '';
	},
	
	HideText: function ()
	{
		if (this._text == null) return;
		var prop = ReadStyle(this.Icon, 'display');
		if (prop == 'none') {
			this._text.className = '';
		}
		else {
			this._text.className = 'wm_hide';
		}
	},
	
	ChangeHandler: function (handler)
	{
		if (handler) this.Cont.onclick = handler;
	},
	
	_build: function (container, desc)
	{
		var div = CreateChildWithAttrs(container, 'span', [['class', 'wm_toolbar_icon'],
			['style', 'background-position: -' + desc.x*X_ICON_SHIFT + 'px -' + desc.y*Y_ICON_SHIFT + 'px']]);
		div.innerHTML = '&nbsp;';
		var titleLangField = (desc.titleLangField) ? desc.titleLangField : desc.langField;
		if (titleLangField) {
			div.title = Lang[titleLangField];
			WebMail.LangChanger.Register('title', div, titleLangField, '');
		}
		if (desc.iconClassName) div.className = desc.iconClassName;
		this.Icon = div;
		
		if (desc.langField) {
			var span = CreateChild(container, 'span');
			span.innerHTML = Lang[desc.langField];
			WebMail.LangChanger.Register('innerHTML', span, desc.langField, '');
			this._text = span;
		}
		
		if (desc.hover == undefined) this._hover = true;
		if (desc.className != undefined) this._className = desc.className;
		if (desc.classNameOver != undefined) this._classNameOver = desc.classNameOver;
	}
};

function CToolBar(parent)
{
	this.table = CreateChild(parent, 'table');
	this.table.className = 'wm_toolbar';
	var tr = this.table.insertRow(0);
	this._container = tr.insertCell(0);
	
	// for Safari and Chrome to not display hidden menus
	document.body.style.cursor = 'auto';
	
	this._descriptions = Array();
	this._descriptions[TOOLBAR_NEW_MESSAGE] = { langField: 'NewMessage', x: 0, y: 0 };
	this._descriptions[TOOLBAR_CHECK_MAIL] = { langField: 'CheckMail', x: 1, y: 0 };
	this._descriptions[TOOLBAR_RELOAD_FOLDERS] = { langField: 'ReloadFolders', x: 2, y: 0 };
	this._descriptions[TOOLBAR_REPLY] = { langField: 'Reply', x: 3, y: 0 };
	this._descriptions[TOOLBAR_REPLYALL] = { langField: 'ReplyAll', x: 4, y: 0, className: 'wm_menu_item', classNameOver: 'wm_menu_item_over' };
	this._descriptions[TOOLBAR_FORWARD] = { langField: 'Forward', x: 5, y: 0 };
	this._descriptions[TOOLBAR_MARK_READ] = { langField: 'MarkAsRead', x: 6, y: 0 };
	this._descriptions[TOOLBAR_MOVE_TO_FOLDER] = { langField: 'MoveToFolder', x: 7, y: 0 };
	this._descriptions[TOOLBAR_DELETE] = { langField: 'Delete', x: 8, y: 0 };
	this._descriptions[TOOLBAR_UNDELETE] = { langField: 'Undelete', x: 9, y: 0, className: 'wm_menu_item wm_delete_menu', classNameOver: 'wm_menu_item_over wm_delete_menu' };
	this._descriptions[TOOLBAR_PURGE] = { langField: 'PurgeDeleted', x: 10, y: 0, className: 'wm_menu_item wm_delete_menu', classNameOver: 'wm_menu_item_over wm_delete_menu' };
	this._descriptions[TOOLBAR_EMPTY_TRASH] = { langField: 'EmptyTrash', x: 11, y: 0, className: 'wm_menu_item wm_delete_menu', classNameOver: 'wm_menu_item_over wm_delete_menu' };
	this._descriptions[TOOLBAR_IS_SPAM] = { langField: 'OperationSpam', x: 12, y: 0 };
	this._descriptions[TOOLBAR_NOT_SPAM] = { langField: 'OperationNotSpam', x: 13, y: 0 };
	this._descriptions[TOOLBAR_SEARCH] = { x: 14, y: 0, iconClassName: 'wm_search_icon', className: 'wm_toolbar_search_item', classNameOver: 'wm_toolbar_search_item_over' };
	this._descriptions[TOOLBAR_BIG_SEARCH] = { x: 15, y: 0 };
	this._descriptions[TOOLBAR_SEARCH_ARROW_DOWN] = { x: 16, y: 0, iconClassName: 'wm_search_icon', className: 'wm_toolbar_search_item', classNameOver: 'wm_toolbar_search_item_over' };
	this._descriptions[TOOLBAR_SEARCH_ARROW_UP] = { x: 17, y: 0, iconClassName: 'wm_search_icon', className: 'wm_toolbar_search_item', classNameOver: 'wm_toolbar_search_item_over' };
	this._descriptions[TOOLBAR_ARROW] = { x: 18, y: 0, iconClassName: 'wm_control_icon' };

	this._descriptions[TOOLBAR_BACK_TO_LIST] = { langField: 'BackToList', x: 0, y: 1 };
	this._descriptions[TOOLBAR_SEND_MESSAGE] = { langField: 'SendMessage', x: 1, y: 1 };
	this._descriptions[TOOLBAR_SAVE_MESSAGE] = { langField: 'SaveMessage', x: 2, y: 1 };
	this._descriptions[TOOLBAR_HIGH_IMPORTANCE] = { langField: 'High', x: 3, y: 1, titleLangField: 'Importance' };
	this._descriptions[TOOLBAR_NORMAL_IMPORTANCE] = { langField: 'Normal', x: 4, y: 1, titleLangField: 'Importance' };
	this._descriptions[TOOLBAR_LOW_IMPORTANCE] = { langField: 'Low', x: 5, y: 1, titleLangField: 'Importance' };
	this._descriptions[TOOLBAR_PRINT_MESSAGE] = { langField: 'Print', x: 6, y: 1 };
	this._descriptions[TOOLBAR_NEXT_ACTIVE] = { x: 7, y: 1, titleLangField: 'NextMsg', iconClassName: 'wm_navigate_icon' };
	this._descriptions[TOOLBAR_NEXT_INACTIVE] = { x: 8, y: 1, titleLangField: 'NextMsg', hover: false, iconClassName: 'wm_navigate_icon' };
	this._descriptions[TOOLBAR_PREV_ACTIVE] = { x: 9, y: 1, titleLangField: 'PreviousMsg', iconClassName: 'wm_navigate_icon' };
	this._descriptions[TOOLBAR_PREV_INACTIVE] = { x: 10, y: 1, titleLangField: 'PreviousMsg', hover: false, iconClassName: 'wm_navigate_icon' };
	this._descriptions[TOOLBAR_NEW_CONTACT] = { langField: 'NewContact', x: 11, y: 1 };
	this._descriptions[TOOLBAR_NEW_GROUP] = { langField: 'NewGroup', x: 12, y: 1 };
	this._descriptions[TOOLBAR_ADD_CONTACTS_TO] = { langField: 'AddContactsTo', x: 13, y: 1 };
	this._descriptions[TOOLBAR_IMPORT_CONTACTS] = { langField: 'ImportContacts', x: 14, y: 1 };
	
	this._descriptions[TOOLBAR_FLAG] = { langField: 'MarkFlag', x: 0, y: 3, className: 'wm_menu_item', classNameOver: 'wm_menu_item_over' };
	this._descriptions[TOOLBAR_UNFLAG] = { langField: 'MarkUnflag', x: 1, y: 3, className: 'wm_menu_item', classNameOver: 'wm_menu_item_over' };
	this._descriptions[TOOLBAR_MARK_ALL_READ] = { langField: 'MarkAllRead', x: 2, y: 3, className: 'wm_menu_item', classNameOver: 'wm_menu_item_over' };
	this._descriptions[TOOLBAR_MARK_ALL_UNREAD] = { langField: 'MarkAllUnread', x: 3, y: 3, className: 'wm_menu_item', classNameOver: 'wm_menu_item_over' };
	this._descriptions[TOOLBAR_MARK_UNREAD] = { langField: 'MarkAsUnread', x: 4, y: 3, className: 'wm_menu_item', classNameOver: 'wm_menu_item_over' };
	
	this._buttons = Array();
	
	this._purgeTool = null;
	this._separatorAll = null;
	this._readAllTool = null;
	this._unreadAllTool = null;
}

CToolBar.prototype = {
	Show: function ()
	{
		this.table.className = 'wm_toolbar';
	},

	Hide: function ()
	{
		this.table.className = 'wm_hide';
	},

	ShowTextLabels: function () {
		var iCount = this._buttons.length;
		for (var i=0; i<iCount; i++) {
			this._buttons[i].ShowText();
		}
	},
	
	HideTextLabels: function () {
		var iCount = this._buttons.length;
		for (var i=0; i<iCount; i++) {
			this._buttons[i].HideText();
		}
	},
	
	AddItem: function(itemId, clickHandler, show) {
		var div = CreateChild(this._container, 'div');
		var button = new CToolButton(div, this._descriptions[itemId], clickHandler, show);
		this._buttons.push(button);
		return button;
	},
	
	AddLastItem: function ()
	{
		var div = CreateChildWithAttrs(this._container, 'div', [['class', 'wm_last_toolbar_item']]);
	},
	
	AddMarkItem: function(popupMenus, show) {
		var markMenu = CreateChild(document.body, 'div');
		markMenu.className = 'wm_hide';
		var markFunc;
		var button;
		for (var i = TOOLBAR_MARK_UNREAD; i <= TOOLBAR_MARK_ALL_UNREAD; i++) {
			var div = CreateChild(markMenu, 'div');
			markFunc = CreateToolBarItemClick(i);
			button = new CToolButton(div, this._descriptions[i], markFunc, true);
			this._buttons.push(button);
			switch (i) {
				case TOOLBAR_UNFLAG:
					div = CreateChild(markMenu, 'div');
					div.className = 'wm_menu_separate';
					this._separatorAll = div;
					break;
				case TOOLBAR_MARK_ALL_READ:
					this._readAllTool = button;
					break;
				case TOOLBAR_MARK_ALL_UNREAD:
					this._unreadAllTool = button;
					break;
			}
		}

		var markReplace = CreateChild(this._container, 'div');
		if (show) markReplace.className = 'wm_tb';
		else markReplace.className = 'wm_hide';

		var markControl;
		if (window.RTL) {
			markControl = CreateChild(markReplace, 'div');
			markControl.className = 'wm_toolbar_item';
			button = new CToolButton(markControl, this._descriptions[TOOLBAR_ARROW], null, true);
			this._buttons.push(button);
		}

		markTitle = CreateChild(markReplace, 'div');
		markTitle.className = 'wm_toolbar_item';
		markFunc = CreateToolBarItemClick(TOOLBAR_MARK_READ);
		button = new CToolButton(markTitle, this._descriptions[TOOLBAR_MARK_READ], markFunc, true);
		this._buttons.push(button);

		if (!window.RTL) {
			markControl = CreateChild(markReplace, 'div');
			markControl.className = 'wm_toolbar_item';
			button = new CToolButton(markControl, this._descriptions[TOOLBAR_ARROW], null, true);
			this._buttons.push(button);
		}

		popupMenus.addItem(markMenu, markControl, 'wm_popup_menu', markReplace, markTitle, 'wm_tb', 'wm_tb_press', 'wm_toolbar_item', 'wm_toolbar_item_over');
		return markReplace;
	},
	
	AddMoveItem: function(id, popupMenus, moveMenu, show) {
		var moveControl = CreateChild(this._container, 'div');
		if (show) moveControl.className = 'wm_toolbar_item';
		else moveControl.className = 'wm_hide';
		var button = new CToolButton(moveControl, this._descriptions[id], null, true);
		this._buttons.push(button);

		if (window.RTL) { var text = CreateTextChild(moveControl, ' '); }
		button = new CToolButton(moveControl, this._descriptions[TOOLBAR_ARROW], null, true);
		this._buttons.push(button);

		popupMenus.addItem(moveMenu, moveControl, 'wm_popup_menu', moveControl, moveControl, 'wm_toolbar_item', 'wm_toolbar_item_press', 'wm_toolbar_item', 'wm_toolbar_item_over');
		return moveControl;
	},
	
	AddReplyItem: function(popupMenus, show) {
		var replyMenu = CreateChild(document.body, 'div');
		replyMenu.className = 'wm_hide';
		var div = CreateChild(replyMenu, 'div');
		var replyFunc = CreateReplyClick(TOOLBAR_REPLYALL);
		var button = new CToolButton(div, this._descriptions[TOOLBAR_REPLYALL], replyFunc, true);
		this._buttons.push(button);

		var replyReplace = CreateChild(this._container, 'div');
		if (show) replyReplace.className = 'wm_tb';
		else replyReplace.className = 'wm_hide';

		var replyControl;
		if (window.RTL) {
			replyControl = CreateChild(replyReplace, 'div');
			var button = new CToolButton(replyControl, this._descriptions[TOOLBAR_ARROW], null, true);
			this._buttons.push(button);
		}
		
		var replyTitle = CreateChild(replyReplace, 'div');
		replyFunc = CreateReplyClick(TOOLBAR_REPLY);
		button = new CToolButton(replyTitle, this._descriptions[TOOLBAR_REPLY], replyFunc, true);
		this._buttons.push(button);
		replyTitle.onclick = CreateReplyClick(TOOLBAR_REPLY);

		if (!window.RTL) {
			replyControl = CreateChild(replyReplace, 'div');
			var button = new CToolButton(replyControl, this._descriptions[TOOLBAR_ARROW], null, true);
			this._buttons.push(button);
		}

		popupMenus.addItem(replyMenu, replyControl, 'wm_popup_menu', replyReplace, replyTitle, 'wm_tb', 'wm_tb_press', 'wm_toolbar_item', 'wm_toolbar_item_over');
		return replyReplace;
	},
	
	AddImapDeleteItem: function(popupMenus, show) {
		var deleteMenu = CreateChild(document.body, 'div');
		deleteMenu.className = 'wm_hide';
		var deleteFunc;
		var button;
		for (var i = TOOLBAR_UNDELETE; i <= TOOLBAR_PURGE; i++) {
			var div = CreateChild(deleteMenu, 'div');
			deleteFunc = CreateToolBarItemClick(i);
			button = new CToolButton(div, this._descriptions[i], deleteFunc, true);
			this._buttons.push(button);
			if (i == TOOLBAR_PURGE) {
				this._purgeTool = button;
			}
		}

		var deleteReplace = CreateChild(this._container, 'div');
		if (show) deleteReplace.className = 'wm_tb';
		else deleteReplace.className = 'wm_hide';
		
		var deleteControl;
		if (window.RTL) {
			deleteControl = CreateChild(deleteReplace, 'div');
			deleteControl.className = 'wm_toolbar_item';
			var button = new CToolButton(deleteControl, this._descriptions[TOOLBAR_ARROW], null, true);
			this._buttons.push(button);
		}

		var deleteTitle = CreateChild(deleteReplace, 'div');
		deleteFunc = CreateToolBarItemClick(TOOLBAR_DELETE);
		button = new CToolButton(deleteTitle, this._descriptions[TOOLBAR_DELETE], deleteFunc, true);
		this._buttons.push(button);

		if (!window.RTL) {
			deleteControl = CreateChild(deleteReplace, 'div');
			deleteControl.className = 'wm_toolbar_item';
			var button = new CToolButton(deleteControl, this._descriptions[TOOLBAR_ARROW], null, true);
			this._buttons.push(button);
		}

		popupMenus.addItem(deleteMenu, deleteControl, 'wm_popup_menu', deleteReplace, deleteTitle, 'wm_tb', 'wm_tb_press', 'wm_toolbar_item', 'wm_toolbar_item_over');
		return deleteReplace;
	},
	
	AddPop3DeleteItem: function(popupMenus, show) {
		var deleteMenu = CreateChild(document.body, 'div');
		deleteMenu.className = 'wm_hide';
		
		var div = CreateChild(deleteMenu, 'div');
		var deleteFunc = CreateToolBarItemClick(TOOLBAR_PURGE);
		var button = new CToolButton(div, this._descriptions[TOOLBAR_EMPTY_TRASH], deleteFunc, true);
		this._buttons.push(button);
		
		var deleteReplace = CreateChild(this._container, 'div');
		if (show) deleteReplace.className = 'wm_tb';
		else deleteReplace.className = 'wm_hide';
		
		var deleteControl;
		if (window.RTL) {
			deleteControl = CreateChild(deleteReplace, 'div');
			var button = new CToolButton(deleteControl, this._descriptions[TOOLBAR_ARROW], null, true);
			this._buttons.push(button);
		}

		var deleteTitle = CreateChild(deleteReplace, 'div');
		deleteFunc = CreateToolBarItemClick(TOOLBAR_DELETE);
		button = new CToolButton(deleteTitle, this._descriptions[TOOLBAR_DELETE], deleteFunc, true);
		this._buttons.push(button);
		
		if (!window.RTL) {
			deleteControl = CreateChild(deleteReplace, 'div');
			var button = new CToolButton(deleteControl, this._descriptions[TOOLBAR_ARROW], null, true);
			this._buttons.push(button);
		}

		popupMenus.addItem(deleteMenu, deleteControl, 'wm_popup_menu', deleteReplace, deleteTitle, 
		'wm_tb', 'wm_tb_press', 'wm_toolbar_item', 'wm_toolbar_item_over');
		return deleteReplace;
	},
	
	CreateSearchButton: function (parent, handler)
	{
		var desc = this._descriptions[TOOLBAR_BIG_SEARCH];
		var span = CreateChildWithAttrs(parent, 'span', [['class', 'wm_search_icon wm_control'],
			['style', 'background-position: -' + desc.x*X_ICON_SHIFT + 'px -' + desc.y*Y_ICON_SHIFT + 'px']]);
		span.innerHTML = '&nbsp;';
		span.onclick = handler;
	},

	AddSearchItems: function() {
		var downControl = CreateChild(this._container, 'div');
		var downButton = new CToolButton(downControl, this._descriptions[TOOLBAR_SEARCH_ARROW_DOWN], null, true);
		if (window.RTL) {
			downButton.Cont.style.marginRight = '0';
		}
		else {
			downButton.Cont.style.marginLeft = '0';
		}
		this._buttons.push(downButton);

		var upControl = CreateChild(this._container, 'div');
		var upButton = new CToolButton(upControl, this._descriptions[TOOLBAR_SEARCH_ARROW_UP], null, true);
		if (window.RTL) {
			upButton.Cont.style.marginRight = '0';
		}
		else {
			upButton.Cont.style.marginLeft = '0';
		}
		this._buttons.push(upButton);

		var smallSearchForm = CreateChild(this._container, 'div');
		if (window.RTL) {
			smallSearchForm.style.marginLeft = '0';
		}
		else {
			smallSearchForm.style.marginRight = '0';
		}
		var lookFor = CreateChildWithAttrs(smallSearchForm, 'input', [['type', 'text'], ['class', 'wm_search_input'], ['maxlength', '255']]);
		var actionButton = new CToolButton(smallSearchForm, this._descriptions[TOOLBAR_SEARCH], null, true);
		this._buttons.push(actionButton);

		return {DownButton: downButton, UpButton: upButton, SmallForm: smallSearchForm, ActionImg: actionButton.Icon, LookFor: lookFor}
	},
	
	DisableInSearch: function (mode) {
		if (mode) {
			this._purgeTool.Hide();
			this._separatorAll.className = 'wm_hide';
			this._readAllTool.Hide();
			this._unreadAllTool.Hide();
		}
		else {
			this._purgeTool.Show();
			this._separatorAll.className = 'wm_menu_separate';
			this._readAllTool.Show();
			this._unreadAllTool.Show();
		}
	}
};

function CFolderParams(sentDraftsType, type, syncType, count, newCount, name) {
	this.SentDraftsType = sentDraftsType;
	this._type = type;
	this._imgType = type;
	this._syncType = syncType;
	this._div = null;
	this.Page = 1;
	this.MsgsCount = count;
	this._newMsgsCount = newCount;
	this.SearchResults = false;
	this._name = name;
	this._skinName = '';
	this._toRemoveCount = 0;
	this._toAppendCount = 0;
	this._unreadedToRemoveCount = 0;
	this._unreadedToAppendCount = 0;
	this._toReadCount = 0;
	this._toUnreadCount = 0;
	this._title = null;
	this._clickHandler = null;
}

CFolderParams.prototype = {
	ChangeImgType: function ()
	{
		if (this._syncType != SYNC_TYPE_NO && this._syncType != SYNC_TYPE_DIRECT_MODE) {
			this._title = Lang.SyncFolder;
			switch (this._type) {
				case FOLDER_TYPE_DEFAULT:
					this._imgType = FOLDER_TYPE_DEFAULT_SYNC;
					break;
				case FOLDER_TYPE_INBOX:
					this._imgType = FOLDER_TYPE_INBOX_SYNC;
					break;
				case FOLDER_TYPE_SENT:
					this._imgType = FOLDER_TYPE_SENT_SYNC;
					break;
				case FOLDER_TYPE_DRAFTS:
					this._imgType = FOLDER_TYPE_DRAFTS_SYNC;
					break;
				case FOLDER_TYPE_TRASH:
					this._imgType = FOLDER_TYPE_TRASH_SYNC;
					break;
			}//switch
		}
	},//ChangeImgType
	
	SetPage: function (page)
	{
		this.Page = page;
	},//SetPage
	
	SetDiv: function (div, skinName, clickHandler)
	{
		this._div = div;
		this._clickHandler = clickHandler;
		if (this._title != null) this._div.title = this._title;
		this._skinName = skinName;
		this.SetFolderNameText();
	},//SetDiv

	GetDiv: function ()
	{
		return this._div;
	},
	
	SetFolderNameText: function ()
	{
		CleanNode(this._div);
		var a = CreateChild(this._div, 'a');
		a.href = '#';
		a.onclick = this._clickHandler;
		
		var span = CreateChild(a, 'span');
		span.className = 'wm_folder_icon';
		span.innerHTML = '&nbsp;';
		var desc = FolderDescriptions[this._imgType];
		span.style.backgroundPosition = '-' + desc.x*X_ICON_SHIFT + 'px -' + desc.y*Y_ICON_SHIFT + 'px';
		
		span = CreateChild(a, 'span');
		var innerHtml = this._name;
		if (this._newMsgsCount != 0) {
			innerHtml += '&nbsp;(<span title="' + Lang.NewMessages + '">' + this._newMsgsCount + '</span>)';
		}
		span.innerHTML = innerHtml + '&nbsp;';
		if (this._syncType == SYNC_TYPE_DIRECT_MODE) {
			span = CreateChild(this._div, 'span');
			span.innerHTML = '&nbsp;' + Lang.DirectAccess + '&nbsp;';
			span.title = Lang.DirectAccessTitle;
			span.className = 'wm_folder_direct_mode';
		}
	},
	
	ChangeMsgsCounts: function (count, newCount, searchResults)
	{
		this.MsgsCount = count;
		this._newMsgsCount = newCount;
		this.SearchResults = searchResults;
		this.SetFolderNameText();
	},//ChangeMsgsCounts
	
	AddToAppend: function (count, unreaded)
	{
		this._toAppendCount += count;
		this._unreadedToAppendCount += unreaded;
	},//AddToAppend

	AddToRemove: function (count, unreaded)
	{
		this._toRemoveCount += count;
		this._unreadedToRemoveCount += unreaded;
	},//AddToRemove
	
	Append: function ()
	{
		this.MsgsCount += this._toAppendCount;
		this._newMsgsCount += this._unreadedToAppendCount;
		this._toAppendCount = 0;
		this._unreadedToAppendCount = 0;
		this.SetFolderNameText();
	},//Append
	
	Remove: function ()
	{
		this.MsgsCount += -this._toRemoveCount;
		this._newMsgsCount += -this._unreadedToRemoveCount;
		this._toRemoveCount = 0;
		this._unreadedToRemoveCount = 0;
		this.SetFolderNameText();
	},//Remove
	
	AddAllToRead: function ()
	{
		this._toReadCount = this._newMsgsCount;
	},//AddAllToRead
	
	AddAllToUnread: function ()
	{
		this._toUnreadCount = this.MsgsCount - this._newMsgsCount;
	},//AddAllToUnread
	
	AddToRead: function (count)
	{
		this._toReadCount += count;
	},//AddToRead
	
	AddToUnread: function (count)
	{
		this._toUnreadCount += count;
	},//AddToUnread
	
	Read: function (count)
	{
		if (count)
			this._newMsgsCount += -count;
		else {
			this._newMsgsCount += -this._toReadCount;
			this._toReadCount = 0;
		};
		if (this._newMsgsCount < 0) this._newMsgsCount = 0;
		this.SetFolderNameText();
	},//Read
	
	Unread: function ()
	{
		this._newMsgsCount += this._toUnreadCount;
		this._toUnreadCount = 0;
		this.SetFolderNameText();
	}//Unread
};

function CFolderLine(fold, td, checkInp, spanIcon, spanA, nameA, nameInp, syncSel, hideImg, skinName, opt, parent, prevIndex, index, upImg, downImg, countTd, sizeTd, allowDirectMode, protocol)
{
	this._protocol = protocol;
	
	this._fold = {};
	this._fold.Id = fold.Id;
	this._fold.IdParent = fold.IdParent;
	this._fold.Type = fold.Type;
	this._fold.SyncType = fold.SyncType;
	this._fold.Hide  = fold.Hide ;
	this._fold.FldOrder = fold.FldOrder;
	this._fold.hasChilds = fold.hasChilds;
	this._fold.MsgCount = fold.MsgCount;
	this._fold.NewMsgCount = fold.NewMsgCount;
	this._fold.Size = fold.Size;
	
	var langField = FolderDescriptions[fold.Type].langField;
	this._fold.Name = (langField != undefined) ? Lang[langField] : fold.Name;
	this._fold.RealName = fold.Name;
	this._fold.FullName = fold.FullName;
	this._fold.Level = fold.Level;
	this._fold.Checked = false;
	this._fold.PrevIndex = prevIndex;
	this._fold.Index = index;
	this._fold.NextIndex = -1;
	
	this._container = td;
	this._checkInp = checkInp;
	this._spanIcon = spanIcon;
	this._nameA = nameA;
	this._spanA = spanA;
	this._nameInp = nameInp;
	this._syncSel = syncSel;
	this._hideImg = hideImg;
	this._skinName = skinName;
	this._opt = opt;
	this._parent = parent;
	this._upImg = upImg;
	this._downImg = downImg;
	this._countTd = countTd;
	this._sizeTd = sizeTd;
	this._allowDirectMode = allowDirectMode;
	this._directModeOpt = null;
	
	this.checkDisable = 0;

	this.Init();
}

CFolderLine.prototype = {
	Init: function () {
		var obj = this;
		
		indent = FOLDERS_TREES_INDENT * this._fold.Level;
		this._container.style.paddingLeft = indent + 'px';
		
		if (this._fold.Type == FOLDER_TYPE_DEFAULT) {
			if (this._protocol != POP3_PROTOCOL && (this._fold.hasChilds || this._fold.MsgCount > 0)) {
				this._checkInp.checked = false;
				this._checkInp.disabled = true;
				this._checkInp.onchange = function () { };
				this.checkDisable = 1;
			}
			else {
				this._checkInp.checked = this._fold.Checked;
				this._checkInp.disabled = false;
				this._checkInp.onchange = function () { obj.SetChecked(this.checked); };
			}
			this._checkInp.className = 'wm_checkbox';
		}
		else {
			this._checkInp.className = 'wm_hide';
		}
		
		var strIndent = '';
		for (var j=0; j<this._fold.Level; j++) strIndent += '&nbsp;&nbsp;&nbsp;&nbsp;';
		this._opt.innerHTML = strIndent + this._fold.Name;
		if (this._fold.Type == FOLDER_TYPE_DEFAULT) {
			this._spanA.innerHTML = '&nbsp;';
			this._nameA.innerHTML = this._fold.Name;
			this._nameA.onclick = function () { obj.EditName(); return false; };
		}
		else {
			this._spanA.innerHTML = '&nbsp;' + this._fold.Name;
			this._nameA.innerHTML = '';
			this._nameA.onclick = function () { return false; };
		}
		var desc = FolderDescriptions[this._fold.Type];
		this._spanIcon.style.backgroundPosition = '-' + desc.x*X_ICON_SHIFT + 'px -' + desc.y*Y_ICON_SHIFT + 'px';
		this._nameInp.onkeyup = function (ev) {
			if (isEnter(ev)) obj.SaveName();
		};
		this._nameInp.onblur = function () { obj.SaveName(); };
		
		this._countTd.innerHTML = this._fold.MsgCount;
		this._sizeTd.innerHTML = GetFriendlySize(this._fold.Size);

		if (this._syncSel != null) {
			this.FillSyncSel();
			this._syncSel.onchange = function () {
				if (!obj._allowDirectMode && obj._directModeOpt != null) {
					obj._syncSel.removeChild(obj._directModeOpt);
					obj._directModeOpt = null;
				}
				obj.SetSyncType(this.value);
			}
		}
			
		if (this._fold.Hide) this._hideImg.src = 'skins/' + this._skinName + '/folders/hide.gif';
		else this._hideImg.src = 'skins/' + this._skinName + '/folders/show.gif';
		this._hideImg.onclick = function () { obj.ChangeHide(); };
		
		if (this._fold.PrevIndex == -1) {
			this._upImg.className = '';
			this._upImg.src = 'skins/' + this._skinName + '/folders/up_inactive.gif';
			this._upImg.onclick = function () {};
		}
		else {
			this._upImg.className = 'wm_control_img';
			this._upImg.src = 'skins/' + this._skinName + '/folders/up.gif';
			var obj = this;
			this._upImg.onclick = function () { obj.ChangeWithPrev(); };
		}
	},
	
	SetNextFoldLine: function (index)
	{
		this._fold.NextIndex = index;
		if (this._fold.NextIndex == -1) {
			this._downImg.className = '';
			this._downImg.src = 'skins/' + this._skinName + '/folders/down_inactive.gif';
			this._downImg.onclick = function () {};
		}
		else {
			this._downImg.className = 'wm_control_img';
			this._downImg.src = 'skins/' + this._skinName + '/folders/down.gif';
			var obj = this;
			this._downImg.onclick = function () { obj.ChangeWithNext(); };
		}
	},
	
	ChangeWithPrev: function ()
	{
		this._parent.ChangeFoldersPlaces(this._fold.PrevIndex, this._fold.Index);
	},
	
	ChangeWithNext: function ()
	{
		this._parent.ChangeFoldersPlaces(this._fold.Index, this._fold.NextIndex);
	},

	SetChecked: function (value) {
		if (this._fold.Type == FOLDER_TYPE_DEFAULT && (this._protocol == POP3_PROTOCOL || !this._fold.hasChilds && this._fold.MsgCount == 0)) {
			this._fold.Checked = value;
			this._checkInp.checked = value;
		}
	},
	
	GetCheckedXml: function () {
		if (this._fold.Type == FOLDER_TYPE_DEFAULT && this._fold.Checked)
		    return '<folder id="' + this._fold.Id + '"><full_name>' + GetCData(this._fold.FullName) + '</full_name></folder>';
		else
			return '';
	},
	
	EditName: function () {
		this._nameA.className = 'wm_hide';
		this._nameInp.value = HtmlDecode(this._fold.Name);
		this._nameInp.className = '';
		this._nameInp.focus();
	},
	
	SaveName: function () {
		var value = this._nameInp.value;
		if (Trim(value).length != 0 && this._fold.Name != value ) {
			var val = new CValidate();
			if (!val.IsCorrectFileName(value)) {
				alert(Lang.WarningCantUpdateFolder);
			}
			else {
				this._fold.Name = value;
				var strIndent = '';
				for (var j=0; j<this._fold.Level; j++) strIndent += '&nbsp;&nbsp;&nbsp;&nbsp;';
				this._opt.innerHTML = strIndent + this._fold.Name;
				this._nameA.innerHTML = HtmlEncode(this._fold.Name);
				this._parent.hasChanges = true;
			}
		};
		this.StopEditName();
	},
	
	StopEditName: function () {
		this._nameInp.className = 'wm_hide';
		this._nameInp.blur();
		this._nameA.className = '';
	},
	
	SetSyncType: function (value) {
		this._fold.SyncType = value - 0;
		this._parent.hasChanges = true;
	},
	
	ChangeHide: function () {
		if (this._fold.Hide) {
			this._fold.Hide = false;
			this._hideImg.src = 'skins/' + this._skinName + '/folders/show.gif';
		}
		else {
			this._fold.Hide = true;
			this._hideImg.src = 'skins/' + this._skinName + '/folders/hide.gif';
		};
		this._parent.hasChanges = true;
	},
	
	GetProperties: function () {
		return this._fold;
	},
	
	SetProperties: function (fold) {
		this._fold = fold;
		this.Init();
		this.SetNextFoldLine(this._fold.NextIndex);
	},
	
	FillSyncSel: function () {
		var sel = this._syncSel;
		CleanNode(sel);
		opt = CreateChildWithAttrs(sel, 'option', [['value', SYNC_TYPE_NO]]); opt.innerHTML = Lang.SyncTypes[SYNC_TYPE_NO];
		if (this._fold.SyncType == SYNC_TYPE_NO) opt.selected = true;
		opt = CreateChildWithAttrs(sel, 'option', [['value', SYNC_TYPE_NEW_HEADERS]]); opt.innerHTML = Lang.SyncTypes[SYNC_TYPE_NEW_HEADERS];
		if (this._fold.SyncType == SYNC_TYPE_NEW_HEADERS) opt.selected = true;
		opt = CreateChildWithAttrs(sel, 'option', [['value', SYNC_TYPE_ALL_HEADERS]]); opt.innerHTML = Lang.SyncTypes[SYNC_TYPE_ALL_HEADERS];
		if (this._fold.SyncType == SYNC_TYPE_ALL_HEADERS) opt.selected = true;
		opt = CreateChildWithAttrs(sel, 'option', [['value', SYNC_TYPE_NEW_MSGS]]); opt.innerHTML = Lang.SyncTypes[SYNC_TYPE_NEW_MSGS];
		if (this._fold.SyncType == SYNC_TYPE_NEW_MSGS) opt.selected = true;
		opt = CreateChildWithAttrs(sel, 'option', [['value', SYNC_TYPE_ALL_MSGS]]); opt.innerHTML = Lang.SyncTypes[SYNC_TYPE_ALL_MSGS];
		if (this._fold.SyncType == SYNC_TYPE_ALL_MSGS) opt.selected = true;
		if (this._allowDirectMode || SYNC_TYPE_DIRECT_MODE == this._fold.SyncType) {
			opt = CreateChildWithAttrs(sel, 'option', [['value', SYNC_TYPE_DIRECT_MODE]]); opt.innerHTML = Lang.SyncTypes[SYNC_TYPE_DIRECT_MODE];
			if (this._fold.SyncType == SYNC_TYPE_DIRECT_MODE) opt.selected = true;
			this._directModeOpt = opt;
		}
	},
	
	GetInXml: function () {
		var attrs = '';
		attrs += ' id="' + this._fold.Id + '"';
		attrs += ' sync_type="' + this._fold.SyncType + '"';
		if (this._fold.Hide) {
			attrs += ' hide="1"';
		}
		else {
			attrs += ' hide="0"';
		}
		attrs += ' fld_order="' + this._fold.FldOrder + '"';
		var nodes = '';
		if (this._fold.Type == FOLDER_TYPE_INBOX || this._fold.Type == FOLDER_TYPE_SENT ||
		 this._fold.Type == FOLDER_TYPE_DRAFTS || this._fold.Type == FOLDER_TYPE_TRASH) {
			nodes += '<name>' + GetCData(this._fold.RealName) + '</name>';
		}
		else {
			nodes += '<name>' + GetCData(this._fold.Name) + '</name>';
		}
		nodes += '<full_name>' + GetCData(this._fold.FullName) + '</full_name>';
		return '<folder' + attrs + '>' + nodes + '</folder>';
	}
};

function CSpaceInfo()
{
	this._enable = true;
	this._mailboxLimit = 0;
	this._mailboxSize = 0;
	this._accountSize = 0;
	
	this._progressBarObj = null;
	this._mailboxUsedObj = null;
	this._accountUsedObj = null;
	
	this.ParseSettings = function (settings)
	{
		this._enable = settings.EnableMailboxSizeLimit;
		this._mailboxLimit = settings.MailBoxLimit;
		this._mailboxSize = settings.MailBoxSize;
		this._accountSize = settings.AccountSize;
		this._fill();
	};
	
	this.SetAccountSize = function (size)
	{
		this._accountSize = size;
		this._fill();
	};
	
	this._checkPercent = function (percent)
	{
		if (percent > 100) return 100;
		if (percent < 0) return 0;
		return percent;
	};
	
	this._fill = function ()
	{
		if (!this._enable || this._mailboxLimit == 0 || !this._builded) return;
		var mailboxPercent = 0;
		var accountWidth = 0;
		if (this._mailboxLimit > 0) {
			mailboxPercent = this._checkPercent(Math.round(this._mailboxSize / this._mailboxLimit * 100));
			accountWidth = this._checkPercent(Math.round(this._accountSize / this._mailboxLimit * 100));
		}
		var mailboxWidth = 0;
		if (mailboxPercent > accountWidth) {
			mailboxWidth = mailboxPercent - accountWidth;
		}
		this._progressBarObj.title = Lang.YouUsing + ' ' + mailboxPercent + '% ' + Lang.OfYour + ' ' + GetFriendlySize(this._mailboxLimit);
		this._mailboxUsedObj.style.width = mailboxWidth + 'px';
		this._accountUsedObj.style.width = accountWidth + 'px';
	};
	
	this.Build = function (container, settings)
	{
		this.ParseSettings(settings);
		if (this._enable && this._mailboxLimit > 0) {
			this._progressBarObj = CreateChild(container, 'span');
			this._progressBarObj.className = 'wm_lowtoolbar_space_info';
			var div = CreateChild(this._progressBarObj, 'div');
			div.className = 'wm_progressbar';

			var usedDiv = CreateChild(div, 'div');
			usedDiv.className = 'wm_progressbar_used';
			this._accountUsedObj = usedDiv;

			var usedDiv = CreateChild(div, 'div');
			usedDiv.className = 'wm_progressbar_all_used';
			this._mailboxUsedObj = usedDiv;

			this._builded = true;
		}
		this._fill();
	}
}

function CContactCard() 
{
	this._contact = null;
	this._contactEmail = '';
	this._contactHEmail = '';
	this._contactBEmail = '';
	this._contactOtherEmail = '';
	this._contactHWeb = '';
	this._contactBWeb = '';

	this._mainCont = null;
	
	this._fullnameObj = null;
	this._fullnameCont = null;
	this._defaultEmailObj = null;
	this._defaultEmailCont = null;
	this._birthdayObj = null;
	this._birthdayCont = null;

	this._builded = false;
	this._pageX = 0;
	this._pageY = 0;
	
	this._sectionClassName = '';
	this._titleClassName = 'wm_line_title';
	this._nameClassName = '';
	this._emailClassName = '';
	this._sectionNameClassName = 'wm_section_name';

	this._showRequest = 0;
	this._shown = false;

	this.SetContact = function (contact)
	{
		this._contact = contact;
	};
	
	this.Go = function ()
	{
		SetHistoryHandler(
			{
				ScreenId: SCREEN_CONTACTS,
				Entity: PART_VIEW_CONTACT,
				IdAddr: ContactCard._contact.Id,
				IdAcct: WebMail._idAcct
			}
		);
		return false;
	};
	
	this.Show = function (e)
	{
		ContactCard._showRequest++;
		if (ContactCard._shown) return;
		ContactCard._build();
		
		e = e ? e : window.event;
		ContactCard._pageX = e.clientX;
		ContactCard._pageY = e.clientY;
		if (Browser.Mozilla) {
			ContactCard._pageX = e.pageX;
			ContactCard._pageY = e.pageY;
		}
		if (Browser.Opera) {
			ContactCard._pageX += document.documentElement.scrollLeft - document.documentElement.clientLeft;
			ContactCard._pageY += document.documentElement.scrollTop - document.documentElement.clientTop;
		}
		setTimeout('ContactCard._show()', 200);
	};
	
	this._show = function ()
	{
		if (ContactCard._showRequest == 0) return;
		ContactCard._mainCont.className = 'wm_contact_card';
		with (ContactCard._mainCont.style) {
			left = ContactCard._pageX + 10 + 'px';
			top = ContactCard._pageY + 10 + 'px';
		}
		ContactCard.FillDefSection(ContactCard._contact);
		ContactCard.FillHomeSection(ContactCard._contact);
		ContactCard.FillBusinessSection(ContactCard._contact);
		ContactCard.FillOtherSection(ContactCard._contact);
		ContactCard._shown = true;
	};
	
	this.Hide = function ()
	{
		ContactCard._showRequest--;
		setTimeout('ContactCard._hide()', 400);
	};
	
	this._hide = function ()
	{
		if (ContactCard._showRequest != 0) return;
		ContactCard._mainCont.className = 'wm_hide';
		ContactCard._shown = false;
	};
	
	this._mailTo = function (email)
	{
		if (ContactCard._contact != null) MailToHandler(email);
		ContactCard._hide();
		ContactCard._showRequest = 0;
	};
	
	this._openUrl = function (url)
	{
		if (ContactCard._contact != null) OpenURL(url);
		ContactCard._hide();
	};
	
	this._build = function ()
	{
		if (this._builded) return;
		this._mainCont = CreateChildWithAttrs(document.body, 'table', [['class', 'wm_contact_card']]);
		this._mainCont.onmouseover = function () { ContactCard._showRequest++; };
		this._mainCont.onmouseout = ContactCard.Hide;
		var tr = this._mainCont.insertRow(0);
		var td = tr.insertCell(0);
		td.style.textAlign = 'left';
		var a = CreateChildWithAttrs(td, 'a', [['href', '#']]);
		a.onclick = function () { ContactCard._mailTo(ContactCard._contactEmail); return false; };
		a.innerHTML = Lang.ContactMail;
		WebMail.LangChanger.Register('innerHTML', a, 'ContactMail', '');
		
		td = tr.insertCell(1);
		td.style.textAlign = 'right';
		a = CreateChildWithAttrs(td, 'a', [['href', '#']]);
		a.onclick = function () {
			if (ContactCard._contact != null) {
				ViewAllContactMailsHandler(ContactCard._contact);
				ContactCard._hide();
			}
			return false;
		};
		a.innerHTML = Lang.ContactViewAllMails;
		WebMail.LangChanger.Register('innerHTML', a, 'ContactViewAllMails', '');
		
		tr = this._mainCont.insertRow(1);
		td = tr.insertCell(0);
		td.colSpan = 2;
		td.className = 'wm_view_sections';
		this.BuildDefSection(td);
		this.BuildHomeSection(td);
		this.BuildBusinessSection(td);
		this.BuildOtherSection(td);
		
		this._defaultEmailObj.onclick = function () { ContactCard._mailTo(ContactCard._contactEmail); return false; };
		this._hEmailObj.onclick = function () { ContactCard._mailTo(ContactCard._contactHEmail); return false; };
		this._bEmailObj.onclick = function () { ContactCard._mailTo(ContactCard._contactBEmail); return false; };
		this._otherEmailObj.onclick = function () { ContactCard._mailTo(ContactCard._contactOtherEmail); return false; };
		this._hWebObj.onclick = function () { ContactCard._openUrl(ContactCard._contactHWeb); return false; };
		this._bWebObj.onclick = function () {ContactCard. _openUrl(ContactCard._contactBWeb); return false; };
		this._builded = true;
	}
}

function CAddToAddressBookImg(_parent, _skinName)
{
	this._fromName = '';
	this._fromEmail = '';
	this._img = null;
	
	this.Show = function (name, email)
	{
		this._fromName = name;
		this._fromEmail = email;
		this._img.className = 'wm_add_address_book_img';
	};
	
	this.Hide = function ()
	{
		this._img.className = 'wm_hide';
	};
	
	this.ChangeSkin = function (skinName)
	{
		_skinName = skinName;
		this._img.src = 'skins/' + _skinName + '/contacts/save.gif';
	};
	
	this._build = function ()
	{
		var img = CreateChildWithAttrs(_parent, 'img', [['class', 'wm_hide'], ['style', 'margin: 0'],
			['src', 'skins/' + _skinName + '/contacts/save.gif'], ['title', Lang.AddToAddressBokk]]);
		WebMail.LangChanger.Register('title', img, 'AddToAddressBokk', '');
		var obj = this;
		img.onclick = function () {
			if (obj._fromName.length == 0 && obj._fromEmail.length == 0) return;
			SetHistoryHandler(
				{
					ScreenId: SCREEN_CONTACTS,
					Entity: PART_NEW_CONTACT,
					Name: obj._fromName,
					Email: obj._fromEmail
				}
			);
		};
		this._img = img;
	};
	
	this._build();
}

function CMessageCharsetSelector(_parent)
{
	this._hasCharset = true;
	this._charset = AUTOSELECT_CHARSET;
	this._full = false;
	
	this.Show = function ()
	{
		this.Fill(this._hasCharset, this._charset, this._full);
	};
	
	this.Hide = function ()
	{
		_parent.innerHTML = '';
	};
	
	this.GetWidth = function ()
	{
		return _parent.clientWidth;
	};
	
	this.Fill = function (hasCharset, charset, full)
	{
		this._hasCharset = hasCharset;
		this._charset = charset;
		this._full = full;
		_parent.innerHTML = '';
		if (this._hasCharset && this._charset == AUTOSELECT_CHARSET) return;
		
		if (full) {
			var font = CreateChild(_parent, 'font');
			font.innerHTML = Lang.Charset + ':';
		}
		var sel = CreateChild(_parent, 'select');
		sel.onchange = function () {
			if (WebMail.ScreenId == SCREEN_MESSAGES_LIST_VIEW) {
				var screen = WebMail.Screens[SCREEN_MESSAGES_LIST_VIEW];
				if (screen) {
					var historyObj = screen.GetCurrMessageHistoryObject();
					historyObj.MsgCharset = this.value;
					SetHistoryHandler(historyObj);
				}
			}
		};
		var opt;
		for (var i in Charsets) {
			var value = (Charsets[i].Value == 0) ? AUTOSELECT_CHARSET : Charsets[i].Value;
			opt = CreateChildWithAttrs(sel, 'option', [['value', value]]);
			opt.innerHTML = Charsets[i].Name;
			opt.selected = (charset == value);
		}
		sel.blur();
	};
}

PreviewPaneMessageHeaders = {
	_container: null,
	_contPadding: 6,
	_spanMargin: 4,
	
	_shortLines: [],
	_fullLines: [],

	_subjCont: null,
	_switchDetailsCont: null,
	_switchDetailsObj: null,

	_shortFromCont: null,
	_shortAddToABCont: null,
	_shortAddToABObj: null,
	_shortDateCont: null,
	_shorCharsetSelector: null,

	_fromCont: null,
	SwitcherCont: null,
	SwitcherObj: null,
	_addToABCont: null,
	_addToABObj: null,
	_toCont: null,
	_dateCont: null,
	_charsetSelector: null,
	_ccCont: null,
	_bccCont: null,
	_copiesCont: null,
	
	_shown: false,
	AllowContacts: false,
	_hasCopies: false,
	_showFull: false,
	
	ShowShort: function ()
	{
		for (var shortIndex = 0; shortIndex < this._shortLines.length; shortIndex++) {
			this._shortLines[shortIndex].className = '';
		}
		for (var fullIndex = 0; fullIndex < this._fullLines.length; fullIndex++) {
			this._fullLines[fullIndex].className = 'wm_hide';
		}
		this._copiesCont.className = 'wm_hide';
		this._switchDetailsCont.className = 'wm_message_right';
		this._switchDetailsObj.innerHTML = Lang.MessageShowDetails;
		this._showFull = false;
	},
	
	ShowFull: function ()
	{
		for (var shortIndex = 0; shortIndex < this._shortLines.length; shortIndex++) {
			this._shortLines[shortIndex].className = 'wm_hide';
		}
		for (var fullIndex = 0; fullIndex < this._fullLines.length; fullIndex++) {
			this._fullLines[fullIndex].className = '';
		}
		this._copiesCont.className = (this._hasCopies) ? '' : 'wm_hide';
		this._switchDetailsCont.className = 'wm_message_right';
		this._switchDetailsObj.innerHTML = Lang.MessageHideDetails;
		this._showFull = true;
	},
	
	SwitchDetails: function ()
	{
		if (this._showFull) this.ShowShort();
		else this.ShowFull();
	},
	
	GetHeight: function ()
	{
		return this._container.offsetHeight;
	},
	
	ChangeSkin: function (skinName)
	{
		this._shortAddToABObj.ChangeSkin(skinName);
		this._addToABObj.ChangeSkin(skinName);
	},
	
	Fill: function (msg, contact)
	{
		this._subjCont.innerHTML = msg.Subject;
		this._shortCharsetSelector.Fill(msg.HasCharset, msg.Charset, true);
		this._charsetSelector.Fill(msg.HasCharset, msg.Charset, true);

		var fromDisplay = (msg.FromDisplayName.length > 0) 
				? msg.FromDisplayName 
				: msg.FromAddr;
		if (contact == null || contact.Id == -1) {
			this._fromCont.innerHTML = '<font>' + Lang.From + ':</font>' + '<span class="wm_message_from">' + msg.FromAddr + '</span>';
            this._shortFromCont.innerHTML = '<span class="wm_message_from">' + fromDisplay + '</span>';
			if (this.AllowContacts && msg.FromAddr.length > 0) {
				//email parts for adding to contacts
				var fromParts = GetEmailParts(HtmlDecode(msg.FromAddr));
				this._addToABObj.Show(fromParts.Name, fromParts.Email);
				this._shortAddToABObj.Show(fromParts.Name, fromParts.Email);
			}
			else {
				this._addToABObj.Hide();
				this._shortAddToABObj.Hide();
			}
		}
		else {
			ContactCard.SetContact(contact);

			this._shortFromCont.innerHTML = '';
			var a = CreateChildWithAttrs(this._shortFromCont, 'a', [['href', '#'], ['class', 'wm_message_from'], 
				['style', 'text-decoration: underline;']]);
			a.onclick = ContactCard.Go;
			if (contact.Type == TYPE_CONTACT) {
				a.onmouseover = ContactCard.Show;
				a.onmouseout = ContactCard.Hide;
			}
			a.innerHTML = fromDisplay;
			this._shortAddToABObj.Hide();
			
			this._fromCont.innerHTML = '';
			var font = CreateChild(this._fromCont, 'font');
			font.innerHTML = Lang.From + ':';
			a = CreateChildWithAttrs(this._fromCont, 'a', [['href', '#'], ['class', 'wm_message_from'], 
				['style', 'text-decoration: underline;']]);
			a.onclick = ContactCard.Go;
			if (contact.Type == TYPE_CONTACT) {
				a.onmouseover = ContactCard.Show;
				a.onmouseout = ContactCard.Hide;
			}
			a.innerHTML = msg.FromAddr;
			this._addToABObj.Hide();
		}

		var innerHtml = '';
		if (msg.ToAddr) innerHtml += Lang.MessageForAddr + ' <span class="wm_message_from">' + msg.ToAddr + '</span>';
		var time = (msg.Date != msg.Time) ? ', ' + msg.Time : '';
		if (msg.Date) innerHtml += ' (' + msg.Date + time + ')';
		this._shortDateCont.innerHTML = innerHtml;

		if (msg.ToAddr) this._toCont.innerHTML = '<font>' + Lang.To + ':</font>' + msg.ToAddr;
		else this._toCont.innerHTML = '<font>' + Lang.To + ':</font>';
		if (msg.FullDate) this._dateCont.innerHTML = '<font>' + Lang.Date + ':</font>' + msg.FullDate;
		else this._dateCont.innerHTML = '<font>' + Lang.Date + ':</font>';

		this._hasCopies = false;
		if (msg.CCAddr) {
			this._ccCont.innerHTML = '<font>' + Lang.CC + ':</font>' + msg.CCAddr;
			this._hasCopies = true;
		}
		if (msg.BCCAddr) {
			this._bccCont.innerHTML = '<font>' + Lang.BCC + ':</font>' + msg.BCCAddr;
			this._hasCopies = true;
		}
		this.ShowShort();
	},
	
	Clean: function ()
	{
		this._subjCont.innerHTML = '';
		this._switchDetailsCont.className = 'wm_hide';
		
		this._shortFromCont.innerHTML = '';
		this._shortAddToABObj.Hide();
		this._shortDateCont.innerHTML = '';
		this._shortCharsetSelector.Hide();

		this._fromCont.innerHTML = '';
		this.SwitcherCont.className = 'wm_hide';
		this._addToABObj.Hide();
		this._toCont.innerHTML = '';
		this._dateCont.innerHTML = '';
		this._charsetSelector.Hide();
		this._ccCont.innerHTML = '';
		this._bccCont.innerHTML = '';
		this._copiesCont.className = 'wm_hide';
	},
	
	Resize: function (inboxWidth)
	{
		var switchDetailsWidth = this._switchDetailsCont.clientWidth;
		switchDetailsWidth += (switchDetailsWidth == 0) ? 0 : this._spanMargin*2;
		var maxSubjWidth = inboxWidth - switchDetailsWidth - this._spanMargin*2 - this._contPadding*2;
		this._subjCont.style.width = 'auto';
		var subjWidth = this._subjCont.clientWidth;
		if (subjWidth > maxSubjWidth) this._subjCont.style.width = maxSubjWidth + 'px';
		
		var charsetWidth = this._shortCharsetSelector.GetWidth();
		charsetWidth += (charsetWidth == 0) ? 0 : this._spanMargin*2;
		var dateWidth = this._shortDateCont.clientWidth;
		dateWidth += (dateWidth == 0) ? 0 : this._spanMargin*2;
		var addrBookWidth = this._shortAddToABCont.clientWidth;
		var maxFromWidth = inboxWidth - charsetWidth - dateWidth - addrBookWidth - this._spanMargin - this._contPadding*2;
		this._shortFromCont.style.width = 'auto';
		var fromWidth = this._shortFromCont.clientWidth;
		if (fromWidth > maxFromWidth) this._shortFromCont.style.width = maxFromWidth + 'px';

		charsetWidth = this._charsetSelector.GetWidth();
		charsetWidth += (charsetWidth == 0) ? 0 : this._spanMargin*2;
		addrBookWidth = this._addToABCont.clientWidth;
		maxFromWidth = inboxWidth - charsetWidth - addrBookWidth - 4 - this._contPadding*2;
		this._fromCont.style.width = 'auto';
		fromWidth = this._fromCont.clientWidth;
		if (fromWidth > maxFromWidth) this._fromCont.style.width = maxFromWidth + 'px';

		dateWidth = this._dateCont.clientWidth;
		dateWidth += (dateWidth == 0) ? 0 : this._spanMargin*2;
		var switcherWidth = this.SwitcherCont.clientWidth;
		switcherWidth += (switcherWidth == 0) ? 0 : this._spanMargin*2;
		var maxToWidth = inboxWidth - dateWidth - switcherWidth - 8 - this._contPadding*2;
		this._toCont.style.width = 'auto';
		var toWidth = this._toCont.clientWidth;
		if (toWidth > maxToWidth) this._toCont.style.width = maxToWidth + 'px';

		if (this._hasCopies) {
			if (this._ccCont.innerHTML == '') {
				this._bccCont.style.width = inboxWidth - this._spanMargin*2 - this._contPadding*2 + 'px';
			}
			else if (this._bccCont.innerHTML == '') {
				this._ccCont.style.width = inboxWidth - this._spanMargin*2 - this._contPadding*2 + 'px';
			}
			else {
				var halfWidth = Math.ceil(inboxWidth / 2) - this._spanMargin*2 - this._contPadding;
				this._ccCont.style.width = 'auto';
				this._bccCont.style.width = 'auto';
				var ccWidth = this._ccCont.clientWidth;
				var bccWidth = this._bccCont.clientWidth;
				if ((ccWidth + bccWidth) > halfWidth * 2) {
					if (ccWidth > halfWidth) {
						if (bccWidth > halfWidth) {
							this._ccCont.style.width = halfWidth + 'px';
							this._bccCont.style.width = halfWidth + 'px';
						}
						else this._ccCont.style.width = halfWidth * 2 - bccWidth + 'px';
					}
					else if (bccWidth > halfWidth) this._bccCont.style.width = halfWidth * 2 - ccWidth + 'px';
				}
			}
		}
	},
	
	Build: function (parent, skinName)
	{
		var cont = CreateChild(parent, 'div');
		cont.style.padding = this._contPadding + 'px';
		cont.style.paddingTop = '4px';
		cont.style.paddingBottom = '4px';
		cont.className = 'wm_message_headers';
		this._container = cont;

		var div = CreateChild(cont, 'div');
		this._subjCont = CreateChildWithAttrs(div, 'span', [['style', 'font-size: 14px; font-weight: bold;'],
			['class', 'wm_message_left wm_message_resized']]);
		this._switchDetailsCont = CreateChildWithAttrs(div, 'span', [['class', 'wm_hide']]);
		var a = CreateChildWithAttrs(this._switchDetailsCont, 'a', [['href', '#']]);
		a.innerHTML = Lang.MessageShowDetails;
		a.onclick = function () {
			PreviewPaneMessageHeaders.SwitchDetails();
			if (WebMail.ScreenId == SCREEN_MESSAGES_LIST_VIEW) {
				var screen = WebMail.Screens[SCREEN_MESSAGES_LIST_VIEW];
				if (screen) screen.ResizeScreen();
			}
			return false;
		};
		this._switchDetailsObj = a;
		div = CreateChildWithAttrs(cont, 'div', [['style', 'width: 0; height: 0; padding: 0; overflow: hidden; clear: both;']]);

		div = CreateChild(cont, 'div');
		this._shortFromCont = CreateChildWithAttrs(div, 'span', [['style', 'margin-right: 0;'], 
			['class', 'wm_message_left wm_message_resized']]);
		this._shortAddToABCont = CreateChildWithAttrs(div, 'span', [['style', 'margin-left: 0; margin-right: 0;'], 
			['class', 'wm_message_left']]);
		this._shortAddToABObj = new CAddToAddressBookImg(this._shortAddToABCont, skinName);
		this._shortDateCont = CreateChildWithAttrs(div, 'span', [['class', 'wm_message_left']]);
		var charsetCont = CreateChildWithAttrs(div, 'span', [['class', 'wm_message_right']]);
		this._shortCharsetSelector = new CMessageCharsetSelector(charsetCont);
		this._shortLines.push(div);
		div = CreateChildWithAttrs(cont, 'div', [['style', 'width: 0; height: 0; padding: 0; overflow: hidden; clear: both;']]);
		
		div = CreateChildWithAttrs(cont, 'div', [['class', 'wm_hide']]);
		this._fromCont = CreateChildWithAttrs(div, 'span', [['style', 'margin-right: 0;'], 
			['class', 'wm_message_left wm_message_resized']]);
		this._addToABCont = CreateChildWithAttrs(div, 'span', [['style', 'margin-left: 0; margin-right: 0;'], 
			['class', 'wm_message_left']]);
		this._addToABObj = new CAddToAddressBookImg(this._addToABCont, skinName);
		charsetCont = CreateChildWithAttrs(div, 'span', [['class', 'wm_message_right']]);
		this._charsetSelector = new CMessageCharsetSelector(charsetCont);
		this._fullLines.push(div);
		div = CreateChildWithAttrs(cont, 'div', [['style', 'width: 0; height: 0; padding: 0; overflow: hidden; clear: both;']]);

		div = CreateChildWithAttrs(cont, 'div', [['class', 'wm_hide']]);
		this._toCont = CreateChildWithAttrs(div, 'span', [['class', 'wm_message_left wm_message_resized']]);
		this._dateCont = CreateChildWithAttrs(div, 'span', [['class', 'wm_message_left']]);
		this.SwitcherCont = CreateChildWithAttrs(div, 'span', [['class', 'wm_message_right']]);
		a = CreateChildWithAttrs(this.SwitcherCont, 'a', [['href', '#']]);
		a.innerHTML = Lang.SwitchToPlain;
		a.onclick = function () {
			if (WebMail.ScreenId == SCREEN_MESSAGES_LIST_VIEW) {
				var screen = WebMail.Screens[SCREEN_MESSAGES_LIST_VIEW];
				if (screen) screen.SwitchToHtmlPlain();
			}
			return false;
		};
		this.SwitcherObj = a;
		this._fullLines.push(div);
		div = CreateChildWithAttrs(cont, 'div', [['style', 'width: 0; height: 0; padding: 0; overflow: hidden; clear: both;']]);

		div = CreateChildWithAttrs(cont, 'div', [['class', 'wm_hide']]);
		this._ccCont = CreateChildWithAttrs(div, 'span', [['class', 'wm_message_left wm_message_resized']]);
		this._bccCont = CreateChildWithAttrs(div, 'span', [['class', 'wm_message_left'],
			['style', 'overflow: hidden;']]);
		this._copiesCont = div;
		div = CreateChildWithAttrs(cont, 'div', [['style', 'width: 0; height: 0; padding: 0; overflow: hidden; clear: both;']]);
	}
};